package cc.fascinated.bat.features.afk.profile;

import cc.fascinated.bat.common.Serializable;
import cc.fascinated.bat.model.BatGuild;
import com.google.gson.Gson;
import lombok.NoArgsConstructor;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Member;
import org.bson.Document;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@NoArgsConstructor
public class AfkProfile extends Serializable {
    private static final String DEFAULT_REASON = "Away";

    /**
     * The AFK users in this guild
     */
    private Map<String, String> afkUsers;

    /**
     * Adds a user to the AFK list
     *
     * @param guild  the guild enable afk mode for
     * @param userId the user ID to add
     * @param reason the reason for being AFK
     */
    public void addAfkUser(BatGuild guild, String userId, String reason) {
        if (afkUsers == null) {
            afkUsers = new HashMap<>();
        }
        afkUsers.put(userId, reason == null ? DEFAULT_REASON : reason);

        Guild discordGuild = guild.getDiscordGuild();
        Member member = discordGuild.getMemberById(userId);
        if (member == null) {
            return;
        }
        try {
            member.modifyNickname("[AFK] " + member.getEffectiveName()).queue();
        } catch (Exception ignored) {
        }
    }

    /**
     * Removes a user from the AFK list
     *
     * @param guild  the guild to remove the user from
     * @param userId the user ID to remove
     */
    public void removeAfkUser(BatGuild guild, String userId) {
        if (afkUsers == null) {
            afkUsers = new HashMap<>();
        }
        afkUsers.remove(userId);

        Guild discordGuild = guild.getDiscordGuild();
        Member member = discordGuild.getMemberById(userId);
        if (member == null) {
            return;
        }
        try {
            member.modifyNickname(member.getEffectiveName().replace("[AFK] ", "")).queue();
        } catch (Exception ignored) {
        }
    }

    /**
     * Gets the reason for being AFK
     *
     * @param userId the user ID to get the reason for
     * @return the reason for being AFK
     */
    public String getAfkReason(String userId) {
        if (afkUsers == null) {
            afkUsers = new HashMap<>();
        }
        return afkUsers.get(userId);
    }

    /**
     * Checks if a user is AFK
     *
     * @param userId the user ID to check
     * @return if the user is AFK
     */
    public boolean isAfk(String userId) {
        if (afkUsers == null) {
            afkUsers = new HashMap<>();
        }
        return afkUsers.containsKey(userId);
    }

    @Override
    public void reset() {
        afkUsers = new HashMap<>();
    }

    @Override
    public void load(Document document, Gson gson) {
        afkUsers = new HashMap<>();
        for (String key : document.keySet()) {
            afkUsers.put(key, document.getString(key));
        }
    }

    @Override
    public Document serialize(Gson gson) {
        Document document = new Document();
        for (String key : afkUsers.keySet()) {
            document.put(key, afkUsers.get(key));
        }
        return document;
    }
}
