package cc.fascinated.bat.features.autorole.command;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.RoleUtils;
import cc.fascinated.bat.features.autorole.profile.AutoRoleProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component("autoroles:add.sub")
@CommandInfo(name = "add", description = "Adds a role to the auto roles list")
public class AddSubCommand extends BatCommand {
    @Autowired
    public AddSubCommand() {
        super.addOptions(new OptionData(OptionType.ROLE, "role", "The role to add", true));
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        AutoRoleProfile profile = guild.getProfile(AutoRoleProfile.class);
        // Check if the guild has reached the maximum auto roles count
        int maxRoleSlots = AutoRoleProfile.getMaxRoleSlots(guild);
        if (profile.getRoleSlotsInUse() >= maxRoleSlots) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("The guild can only have a maximum of %d auto roles"
                            .formatted(maxRoleSlots))
                    .build()).queue();
            return;
        }

        OptionMapping option = event.getOption("role");
        assert option != null;
        Role role = option.getAsRole();

        // Check if the role is already in the auto roles list
        if (profile.hasRole(role.getId())) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("The role %s is already in the auto roles list".formatted(role.getAsMention()))
                    .build()).queue();
            return;
        }

        // Check if the bot has permission to give the role
        if (!RoleUtils.hasPermissionToGiveRole(guild, guild.getDiscordGuild().getSelfMember(), role)) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("I do not have permission to give the role %s".formatted(role.getAsMention()))
                    .build()).queue();
            return;
        }

        // Check if the role is higher than the user adding the role
        if (!RoleUtils.hasPermissionToGiveRole(guild, member, role)) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("You cannot add a role that is higher than you")
                    .build()).queue();
            return;
        }

        // Add the role to the auto roles list
        profile.addRole(role.getId());
        event.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription("You have added %s to the auto roles list".formatted(role.getAsMention()))
                .build()).queue();
    }
}
