package cc.fascinated.bat.features.autorole.command;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.autorole.profile.AutoRoleProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component("autoroles:remove.sub")
@CommandInfo(name = "remove", description = "Removes a role from the auto roles list")
public class RemoveSubCommand extends BatCommand {
    @Autowired
    public RemoveSubCommand() {
        super.addOptions(new OptionData(OptionType.ROLE, "role", "The role to remove", true));
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        AutoRoleProfile profile = guild.getProfile(AutoRoleProfile.class);
        OptionMapping option = event.getOption("role");
        assert option != null;

        Role role = option.getAsRole();
        if (!profile.hasRole(role.getId())) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("The role %s is not in the auto roles list".formatted(role.getAsMention()))
                    .build()).queue();
            return;
        }

        profile.removeRole(role.getId());
        event.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription("Successfully removed the role %s from the auto roles list".formatted(role.getAsMention()))
                .build()).queue();
    }
}
