package cc.fascinated.bat.features.base.commands.utility;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.Category;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedDescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.PasteUtils;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@CommandInfo(
        name = "pastebin",
        description = "Uploads the given text to Paste",
        userInstall = true,
        category = Category.UTILITY
)
public class PastebinCommand extends BatCommand {
    public PastebinCommand() {
        super.addOptions(
                new OptionData(OptionType.STRING, "text", "The text to upload to Paste", true)
        );
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        OptionMapping textOption = event.getOption("text");
        assert textOption != null;
        String text = textOption.getAsString();

        // Upload the text to pastebin
        String url = PasteUtils.uploadPaste(text).getUrl();
        event.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription(new EmbedDescriptionBuilder("The text has been uploaded to Paste!")
                        .appendLine("URL: %s".formatted(url), true)
                        .build())
                .build()).queue();
    }
}
