package cc.fascinated.bat.features.base.commands.utility.lookup;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedDescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.LongUtils;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.DiscordService;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.User;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component("lookup.user:sub")
@CommandInfo(name = "user", description = "Lookup a user")
public class UserSubCommand extends BatCommand {
    public UserSubCommand() {
        super.addOptions(new OptionData(OptionType.STRING, "id", "The id of the user", true));
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        OptionMapping idOption = event.getOption("id");
        if (idOption == null) {
            return;
        }
        String id = idOption.getAsString();
        if (!LongUtils.isLong(id)) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("You need to provide a valid user id")
                            .build())
                    .setEphemeral(true)
                    .queue();
            return;
        }

        DiscordService.JDA.retrieveUserById(id).queue(target -> {
            if (target == null) {
                event.replyEmbeds(EmbedUtils.errorEmbed()
                                .setDescription("User `%s` not found".formatted(id))
                                .build())
                        .setEphemeral(true)
                        .queue();
                return;
            }

            // The flags of the user (eg. Discord Partner, Hypesquad Events, etc.)
            StringBuilder flags = new StringBuilder();
            for (User.UserFlag flag : target.getFlags()) {
                flags.append("`").append(flag.getName()).append("`, ");
            }
            String name = target.getGlobalName() == null ? target.getName() : target.getGlobalName().replaceAll("`", "");
            target.retrieveProfile().queue(profile -> event.replyEmbeds(EmbedUtils.genericEmbed()
                            .setDescription(new EmbedDescriptionBuilder("User Lookup")
                                    .appendLine("Name: `%s`".formatted(name), true)
                                    .appendLine("Username: `%s`".formatted(target.getName()), true)
                                    .appendLine("ID: `%s`".formatted(target.getId()), true)
                                    .appendLine("Flags: %s".formatted(flags.toString().isEmpty() ? "None" : flags.substring(0, flags.length() - 2)), true)
                                    .appendLine("Joined Discord: <t:%s:R>".formatted(target.getTimeCreated().toEpochSecond()), true)
                                    .appendLine("Avatar: %s".formatted(target.getAvatar() == null ? "None"
                                            : "[click here](%s)".formatted(target.getAvatar().getUrl(4096))), true)
                                    .appendLine("Banner: %s".formatted(profile.getBanner() == null ? "None"
                                            : "[click here](%s)".formatted(profile.getBanner().getUrl(4096))), true)
                                    .build())
                            .setThumbnail(target.getAvatar() == null ? null : target.getAvatar().getUrl(4096))
                            .build())
                    .setEphemeral(true)
                    .queue());
        });
    }
}
