package cc.fascinated.bat.features.leveling.command;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedDescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.NumberFormatter;
import cc.fascinated.bat.features.leveling.LevelingFeature;
import cc.fascinated.bat.features.leveling.LevelingProfile;
import cc.fascinated.bat.features.leveling.UserLevel;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.UserService;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component("leveling.current:sub")
@CommandInfo(
        name = "current",
        description = "Shows the current level of the user"
)
public class CurrentSubCommand extends BatCommand {
    private final UserService userService;

    @Autowired
    public CurrentSubCommand(@NonNull UserService userService) {
        this.userService = userService;
        super.addOptions(
                new OptionData(OptionType.USER, "user", "The user to get the level of", false)
        );
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        OptionMapping userOption = event.getOption("user");
        BatUser target = userOption == null ? user : userService.getUser(userOption.getAsUser().getId());
        LevelingProfile profile = guild.getLevelingProfile();
        UserLevel level = profile.getUserLevel(target.getId());

        EmbedDescriptionBuilder description = new EmbedDescriptionBuilder("Current Level");
        description.appendLine("User: %s".formatted(target.getDiscordUser().getAsMention()), true);
        description.appendLine("Level: `%s`".formatted(level.getLevel()), true);
        description.appendLine("XP: `%s`/`%s`".formatted(
                NumberFormatter.format(level.getCurrentXp()),
                NumberFormatter.format(LevelingFeature.getBaseXP(level.getLevel()))
        ), true);
        event.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription(description.build())
                .setThumbnail(target.getDiscordUser().getEffectiveAvatarUrl())
                .build()).queue();
    }
}
