package cc.fascinated.bat.features.minecraft.command.serverwatcher;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.minecraft.MinecraftProfile;
import cc.fascinated.bat.features.minecraft.ServerWatcher;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.stereotype.Component;
import xyz.mcutils.models.server.ServerPlatform;

/**
 * @author Fascinated (fascinated7)
 */
@Component("minecraft-server-watcher.remove:sub")
@CommandInfo(
        name = "remove",
        description = "Remove a server from the server watcher"
)
public class RemoveSubCommand extends BatCommand {
    public RemoveSubCommand() {
        super.addOptions(
                new OptionData(OptionType.STRING, "platform", "The platform of the server to lookup", true)
                        .addChoice("Java", "java")
                        .addChoice("Bedrock", "bedrock"),
                new OptionData(OptionType.STRING, "host", "The host/ip of the server to remove", true)
        );
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        OptionMapping platformOption = event.getOption("platform");
        assert platformOption != null;
        OptionMapping hostOption = event.getOption("host");
        assert hostOption != null;

        String platform = platformOption.getAsString();
        String host = hostOption.getAsString();

        MinecraftProfile profile = guild.getMinecraftProfile();
        ServerWatcher serverWatcher = profile.getServerWatcher(host, ServerPlatform.valueOf(platform.toUpperCase()));
        if (serverWatcher == null) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("The server `%s` is not being watched".formatted(host))
                    .build()).queue();
            return;
        }

        profile.removeServerWatcher(serverWatcher);
        event.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription("The server `%s` has been removed from the server watcher".formatted(host))
                .build()).queue();
    }
}
