package cc.fascinated.bat.features.moderation.command;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.Category;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.Permission;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@CommandInfo(name = "purge", description = "Purge messages from a channel", requiredPermissions = Permission.MESSAGE_MANAGE, category = Category.MODERATION)
public class PurgeCommand extends BatCommand {
    private final long MESSAGE_DELETE_DELAY = TimeUnit.SECONDS.toMillis(10);

    public PurgeCommand() {
        super.addOptions(new OptionData(OptionType.INTEGER, "amount", "The amount of messages to remove", true));
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        OptionMapping amountOption = event.getOption("amount");
        assert amountOption != null;

        int amount = amountOption.getAsInt();
        if (amount < 2 || amount > 100) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("You can only purge between 2 and 100 messages")
                    .build()).queue();
            return;
        }

        event.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription("Purging `%s` messages...".formatted(amount))
                .build()).queue(then -> {
            TextChannel textChannel = (TextChannel) channel;
            textChannel.getHistory().retrievePast(amount + 1).queue(messages -> {
                // Filter out the command message
                then.retrieveOriginal().queue(original -> {
                    if (original == null) return;
                    List<Message> toRemove = messages.stream().filter(message -> !original.getId().equals(message.getId())).toList();
                    textChannel.deleteMessages(toRemove).queue(done -> then.editOriginalEmbeds(EmbedUtils.successEmbed()
                            .setDescription("Successfully purged `%s` messages\n\n*This message will be removed <t:%s:R>*".formatted(
                                    amount,
                                    (System.currentTimeMillis() + MESSAGE_DELETE_DELAY) / 1000
                            ))
                            .build()).queue(message -> message.delete().queueAfter(MESSAGE_DELETE_DELAY, TimeUnit.MILLISECONDS)));
                });
            });
        });
    }
}
